@@ -21,13 +21,13 @@ Mail::SpamAssassin::Client - Client for spamd Protocol
 
 =head1 SYNOPSIS
 
-  my $client = new Mail::SpamAssassin::Client({
+  my $client = Mail::SpamAssassin::Client->new({
                                 port => 783,
                                 host => 'localhost',
                                 username => 'someuser'});
   or
 
-  my $client = new Mail::SpamAssassin::Client({
+  my $client = Mail::SpamAssassin::Client->new({
                                 socketpath => '/path/to/socket',
                                 username => 'someuser'});
 
@@ -60,6 +60,17 @@ use re 'taint';
 use IO::Socket;
 use Errno qw(EBADF);
 
+our($io_socket_module_name);
+BEGIN {
+  if (eval { require IO::Socket::IP }) {
+    $io_socket_module_name = 'IO::Socket::IP';
+  } elsif (eval { require IO::Socket::INET6 }) {
+    $io_socket_module_name = 'IO::Socket::INET6';
+  } elsif (eval { require IO::Socket::INET }) {
+    $io_socket_module_name = 'IO::Socket::INET';
+  }
+}
+
 my $EOL = "\015\012";
 my $BLANK = $EOL x 2;
 my $PROTOVERSION = 'SPAMC/1.5';
@@ -95,6 +106,10 @@ sub new {
     $self->{username} = $args->{username};
   }
 
+  if ($args->{max_size}) {
+    $self->{max_size} = $args->{max_size};
+  }
+
   if ($args->{timeout}) {
     $self->{timeout} = $args->{timeout} || 30;
   }
@@ -124,6 +139,8 @@ threshold
 
 message
 
+report
+
 =cut
 
 sub process {
@@ -139,6 +156,41 @@ sub process {
   return $self->_filter($msg, $command);
 }
 
+=head2 spam_report
+
+public instance (\%) spam_report (String $msg)
+
+Description:
+The method implements the report call.
+
+See the process method for the return value.
+
+=cut
+
+sub spam_report {
+  my ($self, $msg) = @_;
+
+  return $self->_filter($msg, 'REPORT');
+}
+
+=head2 spam_report_ifspam
+
+public instance (\%) spam_report_ifspam (String $msg)
+
+Description:
+The method implements the report_ifspam call.
+A report will be returned only if the message is spam.
+
+See the process method for the return value.
+
+=cut
+
+sub spam_report_ifspam {
+  my ($self, $msg) = @_;
+
+  return $self->_filter($msg, 'REPORT_IFSPAM');
+}
+
 =head2 check
 
 public instance (\%) check (String $msg)
@@ -195,7 +247,7 @@ sub learn {
 
   my $remote = $self->_create_connection();
 
-  return undef unless ($remote);
+  return unless $remote;
 
   my $msgsize = length($msg.$EOL);
 
@@ -217,7 +269,7 @@ sub learn {
   else { # bad learntype
     $self->{resp_code} = 00;
     $self->{resp_msg} = 'do not know';
-    return undef;
+    return;
   }
 
   print $remote "$EOL";
@@ -229,14 +281,14 @@ sub learn {
   defined $line || $!==0  or
     $!==EBADF ? dbg("error reading from spamd (1): $!")
               : die "error reading from spamd (1): $!";
-  return undef unless (defined $line);
+  return unless defined $line;
 
   my ($version, $resp_code, $resp_msg) = $self->_parse_response_line($line);
 
   $self->{resp_code} = $resp_code;
   $self->{resp_msg} = $resp_msg;
 
-  return undef unless ($resp_code == 0);
+  return unless $resp_code == 0;
 
   my $did_set = '';
   my $did_remove = '';
@@ -259,10 +311,10 @@ sub learn {
   close $remote  or die "error closing socket: $!";
 
   if ($learntype == 0 || $learntype == 1) {
-    return $did_set =~ /local/;
+    return index($did_set, 'local') >= 0;
   }
   else { #safe since we've already checked the $learntype values
-    return $did_remove =~ /local/;
+    return index($did_remove, 'local') >= 0;
   }
 }
 
@@ -282,7 +334,7 @@ sub report {
 
   my $remote = $self->_create_connection();
 
-  return undef unless ($remote);
+  return unless $remote;
 
   my $msgsize = length($msg.$EOL);
 
@@ -299,14 +351,14 @@ sub report {
   defined $line || $!==0  or
     $!==EBADF ? dbg("error reading from spamd (3): $!")
               : die "error reading from spamd (3): $!";
-  return undef unless (defined $line);
+  return unless defined $line;
 
   my ($version, $resp_code, $resp_msg) = $self->_parse_response_line($line);
 
   $self->{resp_code} = $resp_code;
   $self->{resp_msg} = $resp_msg;
 
-  return undef unless ($resp_code == 0);
+  return unless $resp_code == 0;
 
   my $reported_p = 0;
 
@@ -343,7 +395,7 @@ sub revoke {
 
   my $remote = $self->_create_connection();
 
-  return undef unless ($remote);
+  return unless $remote;
 
   my $msgsize = length($msg.$EOL);
 
@@ -361,14 +413,14 @@ sub revoke {
   defined $line || $!==0  or
     $!==EBADF ? dbg("error reading from spamd (5): $!")
               : die "error reading from spamd (5): $!";
-  return undef unless (defined $line);
+  return unless defined $line;
 
   my ($version, $resp_code, $resp_msg) = $self->_parse_response_line($line);
 
   $self->{resp_code} = $resp_code;
   $self->{resp_msg} = $resp_msg;
 
-  return undef unless ($resp_code == 0);
+  return unless $resp_code == 0;
 
   my $revoked_p = 0;
 
@@ -415,7 +467,7 @@ sub ping {
     $!==EBADF ? dbg("error reading from spamd (7): $!")
               : die "error reading from spamd (7): $!";
   close $remote  or die "error closing socket: $!";
-  return undef unless (defined $line);
+  return unless defined $line;
 
   my ($version, $resp_code, $resp_msg) = $self->_parse_response_line($line);
   return 0 unless ($resp_msg eq 'PONG');
@@ -449,16 +501,17 @@ sub _create_connection {
 				   );
   }
   else {
-    $remote = IO::Socket::INET->new( Proto     => "tcp",
-				     PeerAddr  => $self->{host},
-				     PeerPort  => $self->{port},
-				     Timeout   => $self->{timeout},
-				   );
+    my %params = ( Proto    => "tcp",
+		   PeerAddr => $self->{host},
+		   PeerPort => $self->{port},
+		   Timeout  => $self->{timeout},
+		 );
+    $remote = $io_socket_module_name->new(%params);
   }
 
   unless ($remote) {
-    print "Failed to create connection to spamd daemon: $!\n";
-    return undef;
+    warn "Failed to create connection to spamd daemon: $!\n";
+    return;
   }
 
   $remote;
@@ -522,12 +575,15 @@ threshold
 
 message (if available)
 
+report (if available)
+
 =cut
 
 sub _filter {
   my ($self, $msg, $command) = @_;
 
   my %data;
+  my $msgsize;
 
   $self->_clear_errors();
 
@@ -535,7 +591,10 @@ sub _filter {
 
   return 0 unless ($remote);
 
-  my $msgsize = length($msg.$EOL);
+  if(defined $self->{max_size}) {
+    $msg = substr($msg,0,$self->{max_size});
+  }
+  $msgsize = length($msg);
 
   print $remote "$command $PROTOVERSION$EOL";
   print $remote "Content-length: $msgsize$EOL";
@@ -548,14 +607,14 @@ sub _filter {
   defined $line || $!==0  or
     $!==EBADF ? dbg("error reading from spamd (8): $!")
               : die "error reading from spamd (8): $!";
-  return undef unless (defined $line);
+  return unless defined $line;
 
   my ($version, $resp_code, $resp_msg) = $self->_parse_response_line($line);
   
   $self->{resp_code} = $resp_code;
   $self->{resp_msg} = $resp_msg;
 
-  return undef unless ($resp_code == 0);
+  return unless $resp_code == 0;
 
   for ($!=0; defined($line=<$remote>); $!=0) {
     local($1,$2,$3);
@@ -583,7 +642,11 @@ sub _filter {
     $!==EBADF ? dbg("error reading from spamd (10): $!")
               : die "error reading from spamd (10): $!";
 
-  $data{message} = $return_msg if ($return_msg);
+  if($command =~ /^REPORT/) {
+    $data{report} = $return_msg if ($return_msg);
+  } else {
+    $data{message} = $return_msg if ($return_msg);
+  }
 
   close $remote  or die "error closing socket: $!";
 
