@@ -36,6 +36,7 @@ use strict;
 use warnings;
 use re 'taint';
 
+use Encode qw();
 use Mail::SpamAssassin;
 use Mail::SpamAssassin::Constants qw(:sa);
 use Mail::SpamAssassin::HTML;
@@ -43,13 +44,35 @@ use Mail::SpamAssassin::Logger;
 
 our($enc_utf8, $enc_w1252, $have_encode_detector);
 BEGIN {
-  eval { require Encode }
-    and do { $enc_utf8  = Encode::find_encoding('UTF-8');
-             $enc_w1252 = Encode::find_encoding('Windows-1252') };
+  $enc_utf8  = Encode::find_encoding('UTF-8');
+  $enc_w1252 = Encode::find_encoding('Windows-1252');
   eval { require Encode::Detect::Detector }
     and do { $have_encode_detector = 1 };
 };
 
+# Map of file extensions to MIME types
+my %file_type_map = (
+    'bmp'   => 'image/bmp',
+    'eml'   => 'message/rfc822',
+    'gif'   => 'image/gif',
+    'htm'   => 'text/html',
+    'html'  => 'text/html',
+    'jfif'  => 'image/jpeg',
+    'jpeg'  => 'image/jpeg',
+    'jpg'   => 'image/jpeg',
+    'png'   => 'image/png',
+    'shtml' => 'text/html',
+    'svg'   => 'image/svg+xml',
+    'tif'   => 'image/tiff',
+    'tiff'  => 'image/tiff',
+    'webp'  => 'image/webp',
+);
+
+# Map of MIME type aliases
+my %mime_type_map = (
+    'text/x-amp-html' => 'text/html',
+);
+
 =item new()
 
 Generates an empty Node object and returns it.  Typically only called
@@ -493,7 +516,7 @@ sub _normalize {
   # Try first as UTF-8 ignoring declaring?
   my $tried_utf8;
   if ($cnt_8bits && !$insist_on_declared_charset) {
-    if (eval { $rv = $enc_utf8->decode($_[0], 1|8); defined $rv }) {
+    if (eval { $rv = $enc_utf8->decode($_[0], Encode::FB_CROAK | Encode::LEAVE_SRC); defined $rv }) {
       dbg("message: decoded as charset UTF-8, declared %s",
         $charset_declared);
       return $_[0]  if !$return_decoded;
@@ -523,10 +546,10 @@ sub _normalize {
 
     my $decoder = detect_utf16( $_[0] );
     if (defined $decoder) {
-      if (eval { $rv = $decoder->decode($_[0], 1|8); defined $rv }) {
+      if (eval { $rv = $decoder->decode($_[0], Encode::FB_CROAK | Encode::LEAVE_SRC); defined $rv }) {
         dbg("message: decoded as charset %s, declared %s",
           $decoder->name, $charset_declared);
-        return $_[0]  if !$return_decoded;
+        utf8::encode($rv) if !$return_decoded;
         $rv .= $data_taint;  # carry taintedness over, avoid Encode bug
         return $rv;  # decoded
       } else {
@@ -574,12 +597,9 @@ sub _normalize {
       dbg("message: failed decoding, no decoder for a declared charset %s",
           $chset);
     }
-    elsif ($tried_utf8 && $chset eq 'UTF-8') {
-      # was already tried initially, no point doing again
-    }
     else {
-      my $check_flags = Encode::LEAVE_SRC;  # 0x0008
-      $check_flags |= Encode::FB_CROAK  unless $insist_on_declared_charset;
+      my $check_flags = Encode::LEAVE_SRC;
+      $check_flags |= Encode::FB_CROAK  unless $insist_on_declared_charset || ($tried_utf8 && $chset eq 'UTF-8');
       my $err = '';
       if (eval { $rv = $decoder->decode($_[0], $check_flags); defined $rv }) {
         dbg("message: decoded as charset %s, declared %s",
@@ -616,7 +636,7 @@ sub _normalize {
     # NBSP (A0) and SHY (AD) are at the same position in ISO-8859-* too
     # consider also: AE (r), 80 Euro
     my $err = '';
-    eval { $rv = $enc_w1252->decode($_[0], 1|8) };  # FB_CROAK | LEAVE_SRC
+    eval { $rv = $enc_w1252->decode($_[0], Encode::FB_CROAK | Encode::LEAVE_SRC) };
     if ($@) {
       $err = $@; $err =~ s/\s+/ /gs; $err =~ s/(.*) at .*/$1/;
       $err = " ($err)";
@@ -649,7 +669,7 @@ sub _normalize {
             $charset_detected);
       } else {
         my $err = '';
-        eval { $rv = $decoder->decode($_[0], 1|8) };  # FB_CROAK | LEAVE_SRC
+        eval { $rv = $decoder->decode($_[0], Encode::FB_CROAK | Encode::LEAVE_SRC) };
         if ($@) {
           $err = $@; $err =~ s/\s+/ /gs; $err =~ s/(.*) at .*/$1/;
           $err = " ($err)";
@@ -690,8 +710,13 @@ sub _normalize {
 sub effective_type {
   my ($self) = @_;
   if (!exists $self->{'effective_type'}) {
-    if (($self->{'name'}||'') =~ /\.s?html?$/i) {
-      $self->{'effective_type'} = 'text/html';
+    my $name = $self->{'name'} // '';
+    my $ext;
+    $ext = lc $1 if $name =~ /\.([^.]+)$/;
+    if ( defined $ext && exists $file_type_map{$ext} ) {
+      $self->{'effective_type'} = $file_type_map{$ext};
+    } elsif ( exists $mime_type_map{$self->{'type'}} ) {
+      $self->{'effective_type'} = $mime_type_map{$self->{'type'}};
     } else {
       $self->{'effective_type'} = $self->{'type'};
     }
@@ -760,7 +785,7 @@ sub rendered {
       } else { # non-ASCII, try UTF-8
         my $rv;
         # with some luck input can be interpreted as UTF-8
-        if (eval { $rv = $enc_utf8->decode($text, 1|8); defined $rv }) {
+        if (eval { $rv = $enc_utf8->decode($text, Encode::FB_CROAK | Encode::LEAVE_SRC); defined $rv }) {
           $text = $rv;  # decoded to perl characters
           $character_semantics = 1;  # $text will be in characters
           dbg("message: decoded as charset UTF-8, declared %s", $charset);
@@ -822,7 +847,7 @@ sub rendered {
       if ($text =~ tr/\x00-\x7F//c) {  # non-ASCII, try UTF-8
         my $rv;
         # with some luck input can be interpreted as UTF-8
-        if (eval { $rv = $enc_utf8->decode($text, 1|8); defined $rv }) {
+        if (eval { $rv = $enc_utf8->decode($text, Encode::FB_CROAK | Encode::LEAVE_SRC); defined $rv }) {
           $text = $rv;  # decoded to perl characters
           dbg("message: decoded as charset UTF-8, declared %s", $charset);
         } else {
